package net.sf.statscm;

/*
 * Copyright 2006 Doug Culnane
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import net.sf.statcvs.output.ConfigurationException;

import org.apache.maven.model.Build;
import org.apache.maven.model.Developer;
import org.apache.maven.model.Model;
import org.apache.maven.model.Scm;
import org.apache.maven.model.IssueManagement;
import org.apache.maven.project.MavenProject;

import junit.framework.TestCase;

/**
 * Tests StatXXX Configuration form Maven Project.
 * 
 * @author DoCulnan
 * 
 */
public class StatConfTest extends TestCase
{
    
    /**
     * Check i18n is there.
     *  
     * @throws Exception
     */
    public void testMessageResources() throws Exception
    {
        StatScmMojo statScmMojo = new StatScmMojo();
      
        // If not specified the default Locale should be english
        assertEquals( Locale.ENGLISH.toString(),
                     statScmMojo.getStatConf().getLocale().getLanguage() );
        
        // This message should exist.  ie resouces found.
        assertNotNull( statScmMojo.getMessage( "error.config.no_scm_in_pom" ) );
        
        // Check German resouces found.
        statScmMojo.getStatConf().setLocale( Locale.GERMANY );
        assertEquals("SCM Art wird nicht unterst\u00fctzt.", 
                     statScmMojo.getMessage( "warn.scm.type_xxx" ));
        
    }
    
    /**
     * Test Extract Connection Type.
     * 
     */
    public void testExtractConnectionType()
    {
        assertEquals( StatConf.CONNECTION_TYPE_CVS, StatConf.extractConnectionType( "scm:cvs:..." ) );
        assertEquals(
                      StatConf.CONNECTION_TYPE_CVS,
                      StatConf.extractConnectionType( "scm:cvs:pserver:anonymous@statcvs.cvs.sourceforge.net:/cvsroot/statcvs/statcvs" ) );

        assertEquals( StatConf.CONNECTION_TYPE_SVN,
                      StatConf.extractConnectionType( "svn:https://svn.sourceforge.net/svnroot/stat-scm/trunk" ) );
    }

    /**
     * Test to make sure that null pointers are handled, and errors are throwen corectly.
     * 
     */
    public void testConfugureFromPOM()
    {

        MavenProject project = new MavenProject( new Model() );
        StatScmMojo statScmMojo = new StatScmMojo();
        try
        {
            statScmMojo.getStatConf().configure( project, Locale.ENGLISH );
            fail( "The above should throw an error." );
        }
        catch ( ConfigurationException e )
        {
            assertEquals( e.getMessage(), statScmMojo.getMessage( "error.config.no_scm_in_pom" ) );
        }
        Scm scm = new Scm();
        project.setScm( scm );
        try
        {
            statScmMojo.getStatConf().configure( project, Locale.ENGLISH );
            fail( "This should throw an error." );
        }
        catch ( ConfigurationException e )
        {
            assertEquals( e.getMessage(),
                          statScmMojo.getMessage( "error.config.no_scm_connection_url_in_pom" ) );
        }
        scm.setConnection( "" );
        try
        {
            statScmMojo.getStatConf().configure( project, Locale.ENGLISH );
            fail( "This should throw an error." );
        }
        catch ( ConfigurationException e )
        {
            assertEquals( e.getMessage(),
                          statScmMojo.getMessage( "error.config.no_scm_connection_url_in_pom" ) );
        }
        scm.setConnection( "" );
        try
        {
            statScmMojo.getStatConf().configure( project, Locale.ENGLISH );
            fail( "This should throw an error." );
        }
        catch ( ConfigurationException e )
        {
            assertEquals( e.getMessage(),
                          statScmMojo.getMessage( "error.config.no_scm_connection_url_in_pom" ) );
        }
        scm.setDeveloperConnection( "svn:https://svn.sourceforge.net/svnroot/stat-scm/trunk" );
        try
        {
            statScmMojo.getStatConf().configure( project, Locale.ENGLISH );
            assertEquals( StatConf.CONNECTION_TYPE_SVN, statScmMojo.getStatConf().getConnectionType() );
        }
        catch ( ConfigurationException e )
        {
            fail( "This should not throw an error." );
        }
    }

    /**
     * 
     * @throws ConfigurationException
     */
    public void testScmTypeRecognition() throws ConfigurationException
    {
        MavenProject project = new MavenProject( new Model() );
        StatConf statConf = new StatConf();
        Scm scm = new Scm();
        project.setScm( scm );

        scm.setConnection( "scm:cvs:pserver:anonymous@statcvs.cvs.sourceforge.net:/cvsroot/statcvs/statcvs" );
        statConf.configure( project, Locale.ENGLISH );
        assertTrue( statConf.isStatCVS() );
        assertFalse( statConf.isStatSVN() );
        assertEquals( "cvs", statConf.getConnectionType() );

        scm.setConnection( "svn:https://svn.sourceforge.net/svnroot/stat-scm/trunk" );
        statConf.configure( project, Locale.ENGLISH );
        assertFalse( statConf.isStatCVS() );
        assertTrue( statConf.isStatSVN() );

        assertEquals( "svn", statConf.getConnectionType() );
    }

    /**
     * Test that the Buzilla integration is configured.
     * 
     * @throws ConfigurationException
     */
    public void testSetBugZilla() throws ConfigurationException
    {
        
        String url = "http://sourceforge.net/tracker/?group_id=182522";
        StatConf statConf = new StatConf(); 
        MavenProject project = new MavenProject( new Model() );
        Scm scm = new Scm();
        project.setScm( scm );
        scm.setConnection("scm:svn:http....");
        IssueManagement im = new IssueManagement();
        im.setSystem( "none" );
        im.setUrl( url );
        project.setIssueManagement( im );
        statConf.configure( project, Locale.ENGLISH );
        // assertEqulas( "", StatConf.getBugzillaUrl() );
        im.setSystem( "bugzilla" );
        statConf.configure( project, Locale.ENGLISH );
        // assertEqulas( url, StatConf.getBugzillaUrl() );
    }
 
    /**
     * 
     * @throws ConfigurationException
     */
    public void testConfugureViewVC() throws ConfigurationException
    {
        StatConf statConf = new StatConf();
        MavenProject project = new MavenProject( new Model() );
        Scm scm = new Scm();
        project.setScm( scm );
        scm.setConnection( "scm:svn:https://svn.sourceforge.net/svnroot/stat-scm/trunk" );
        scm.setUrl( "http://stat-scm.svn.sourceforge.net/viewvc/stat-scm/trunk/" );
        statConf.configure( project, Locale.ENGLISH );
        assertEquals( "net.sf.statcvs.output.ViewVcIntegration", StatConf.getWebRepository().getClass().getName() );
    }

    public final void testConfigureDevelopers() throws Exception
    {
        StatConf statConf = new StatConf();

        Scm scm = new Scm();
        Build build = new Build();
        MavenProject project = new MavenProject( new Model() );

        build.setDirectory( "./" );
        scm.setConnection( "scm:svn:https://svn.sourceforge.net/svnroot/stat-scm/trunk" );

        String theId = "1";
        String theName = "ONE";
        Developer dev1 = new Developer();
        dev1.setId( theId );
        dev1.setName( theName );

        List developers = new ArrayList();
        developers.add( dev1 );
        project.setDevelopers( developers );

        project.setScm( scm );
        project.setBuild( build );

        statConf.configure( project, Locale.ENGLISH );

        assertEquals( "System property not set for developer name", theName, System.getProperty( "user." + theId + ".realName" ) );
    }
}
